Improve state-macro encapsulation.
[open-adventure.git] / advent.h
index 8150aa25accacfca6526ddcb41303f610f0e5f99..4188b1707d22616dfddb8e144ed1e3ae5184d29f 100644 (file)
--- a/advent.h
+++ b/advent.h
 #define IS_FIXED -1
 #define IS_FREE 0
 
-/* Map a state property value to a negative range, where the object cannot be
- * picked up but the value can be recovered later.  Avoid colliding with -1,
- * which has its own meaning as STATE_NOTFOUND. */
-#define STASHED(obj)   (-1 - game.prop[obj])
+#ifndef FOUNDBOOL
+/* (ESR) It is fitting that translation of the original ADVENT should
+ * have left us a maze of twisty little conditionals that resists all
+ * understanding.  Setting and use of what is now the per-object state
+ * member (which used to be an array of its own) is our mystery. This
+ * state tangles together information about whether the object is a
+ * treasure, whether the player has seen it yet, and its activation
+ * state.
+ *
+ * Things we think we know:
+ *
+ * STATE_NOTFOUND is only set on treasures. Non-treasures start the
+ * game in STATE_FOUND.
+ *
+ * PROP_STASHED is supposed to map a state property value to a
+ * negative range, where the object cannot be picked up but the value
+ * can be recovered later.  Various objects get this peoperty when
+ * the cave starts to close. On;y seems to be signifucant for the bird 
+ * and readable objects, notably the clam/oyster - but the code around
+ * those test is difficult to read. */
+#define PROP_STASHIFY(n)       (-1 - (n))
+#define PROP_IS_STASHED(obj)   (game.objects[obj].prop < STATE_NOTFOUND)
+#define PROP_IS_NOTFOUND(obj)  (game.objects[obj].prop == STATE_NOTFOUND)
+#define PROP_IS_FOUND(obj)     (game.objects[obj].prop == STATE_FOUND)
+#define PROP_IS_STASHED_OR_UNSEEN(obj) (game.objects[obj].prop < 0)
+#define PROP_SET_FOUND(obj)    (game.objects[obj].prop = STATE_FOUND)
+#define PROP_SET_NOT_FOUND(obj)        (game.objects[obj].prop = STATE_NOTFOUND)
+#define PROP_IS_NOTFOUND2(g, o)        (g.objects[o].prop == STATE_NOTFOUND)
+#define PROP_IS_INVALID(val)   (val < -MAX_STATE - 1 || val > MAX_STATE)
+#else
+/* (ESR) Only the boldest of adventurers will explore here.  This
+ * alternate set of definitions for the macros above was an attempt to
+ * break from out of the state encoding a per-object "found" member
+ * telling whether or not the player has seen the object. 
+ *
+ * What's broken when you try to use thus is
+ * PROP_IS_STASHED_OR_UNSEEN. The symptom is game.tally getting
+ * decremented on non-treasures.
+ */
+#define PROP_STASHIFY(n)       (-(n))
+#define PROP_IS_STASHED(obj)   (game.objects[obj].prop < 0)
+#define PROP_IS_NOTFOUND(obj)  (!game.objects[obj].found)
+#define PROP_IS_FOUND(obj)     (game.objects[obj].found && game.objects[obj].prop == 0)
+#define PROP_IS_STASHED_OR_UNSEEN(obj) (!game.objects[obj].found || game.objects[obj].prop < 0)
+#define PROP_SET_FOUND(obj)    do {game.objects[obj].found = true; game.objects[obj].prop = STATE_FOUND;} while(0)
+#define PROP_SET_NOT_FOUND(obj)        game.objects[obj].found = false
+#define PROP_IS_NOTFOUND2(g, o)        (!g.objects[o].found)
+#define PROP_IS_INVALID(val)   (val < -MAX_STATE || val > MAX_STATE)
+#define PROP_SET_SEEN(obj)     game.objects[object].found = true
+#endif
+#define PROP_STASHED(obj)      PROP_STASHIFY(game.objects[obj].prop)
 
 #define PROMPT "> "
 
  */
 #define DESTROY(N)   move(N, LOC_NOWHERE)
 #define MOD(N,M)     ((N) % (M))
-#define TOTING(OBJ)  (game.place[OBJ] == CARRIED)
-#define AT(OBJ)      (game.place[OBJ] == game.loc || game.fixed[OBJ] == game.loc)
+#define TOTING(OBJ)  (game.objects[OBJ].place == CARRIED)
+#define AT(OBJ)      (game.objects[OBJ].place == game.loc || game.objects[OBJ].fixed == game.loc)
 #define HERE(OBJ)    (AT(OBJ) || TOTING(OBJ))
 #define CNDBIT(L,N)  (tstbit(conditions[L],N))
-#define LIQUID()     (game.prop[BOTTLE] == WATER_BOTTLE? WATER : game.prop[BOTTLE] == OIL_BOTTLE ? OIL : NO_OBJECT )
+#define LIQUID()     (game.objects[BOTTLE].prop == WATER_BOTTLE? WATER : game.objects[BOTTLE].prop == OIL_BOTTLE ? OIL : NO_OBJECT )
 #define LIQLOC(LOC)  (CNDBIT((LOC),COND_FLUID)? CNDBIT((LOC),COND_OILY) ? OIL : WATER : NO_OBJECT)
 #define FORCED(LOC)  CNDBIT(LOC, COND_FORCED)
-#define DARK(DUMMY)  (!CNDBIT(game.loc,COND_LIT) && (game.prop[LAMP] == LAMP_DARK || !HERE(LAMP)))
+#define DARK(DUMMY)  (!CNDBIT(game.loc,COND_LIT) && (game.objects[LAMP].prop == LAMP_DARK || !HERE(LAMP)))
 #define PCT(N)       (randrange(100) < (N))
 #define GSTONE(OBJ)  ((OBJ) == EMERALD || (OBJ) == RUBY || (OBJ) == AMBER || (OBJ) == SAPPH)
 #define FOREST(LOC)  CNDBIT(LOC, COND_FOREST)
@@ -191,12 +238,23 @@ struct game_t {
        loc_t loc;               // location of dwarves, initially hard-wired in
        loc_t oldloc;            // prior loc of each dwarf, initially garbage
     } dwarves[NDWARVES + 1];
-    loc_t fixed[NOBJECTS + 1];   // fixed location of object (if  not IS_FREE)
+    struct {
+#ifdef FOUNDBOOL
+       bool found;              // has the location of this object bween found?
+#endif
+       loc_t fixed;             // fixed location of object (if not IS_FREE)
+        int prop;                // object state */
+       loc_t place;             // location of object
+    } objects[NOBJECTS + 1];
+    struct { 
+       bool used;               // hints[i].used = true iff hint i has been used.
+#ifndef FOUNDBOOL
+       int lc;                 // hints[i].lc = show int at LOC with cond bit i
+#else
+       int lc;                  // hints[i].lc = show int at LOC with cond bit i
+#endif
+    } hints[NHINTS];
     obj_t link[NOBJECTS * 2 + 1];// object-list links
-    loc_t place[NOBJECTS + 1];   // location of object
-    bool hinted[NHINTS];         // hinted[i] = true iff hint i has been used.
-    int hintlc[NHINTS];          // hintlc[i] = show int at LOC with cond bit i
-    int prop[NOBJECTS + 1];      // object state array */
 };
 
 /*
@@ -272,7 +330,7 @@ extern bool silent_yes_or_no(void);
 extern bool yes_or_no(const char*, const char*, const char*);
 extern void juggle(obj_t);
 extern void move(obj_t, loc_t);
-extern loc_t put(obj_t, loc_t, int);
+extern void put(obj_t, loc_t, int);
 extern void carry(obj_t, loc_t);
 extern void drop(obj_t, loc_t);
 extern int atdwrf(loc_t);